"""Constants for the Axis component."""
import logging

LOGGER = logging.getLogger(__package__)

DOMAIN = "axis"

CONF_CAMERA = "camera"
CONF_EVENTS = "events"
CONF_MODEL = "model"

DEFAULT_TRIGGER_TIME = 0
